@@ -27,7 +27,6 @@
 #include <xapian/visibility.h>
 
 #include "flint_types.h"
-#include "flint_btreeutil.h"
 
 class XAPIAN_VISIBILITY_DEFAULT FlintTable_base {
     public:
@@ -85,7 +84,11 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable_base {
 	}
 
 	/** Write the btree base file to disk. */
-	void write_to_file(const std::string &filename);
+	void write_to_file(const std::string &filename,
+			   char base_letter,
+			   const std::string &tablename,
+			   int changes_fd,
+			   const std::string * changes_tail);
 
 	/* Methods dealing with the bitmap */
 	/** true iff block n was free at the start of the transaction on
@@ -97,6 +100,12 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable_base {
 
 	uint4 next_free_block();
 
+	/** Find the first changed block at or after position *n.
+	 *
+	 *  Returns true if such a block was found, or false otherwise.
+	 */
+	bool find_changed_block(uint4 * n);
+
 	bool block_free_now(uint4 n);
 
 	void calculate_last_block();
@@ -117,7 +126,7 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable_base {
 
 	void extend_bit_map();
 
-	/** Do most of the error handling from unpack_uint() */
+	/** Do most of the error handling from F_unpack_uint() */
 	bool do_unpack_uint(const char **start, const char *end,
 			    uint4 *dest, std::string &err_msg,
 			    const std::string &basename,
