@@ -2,7 +2,9 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2009,2010 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010 Olly Betts
+ * Copyright 2008 Lemur Consulting Ltd
+ * Copyright 2010 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,6 +24,8 @@
 
 #include <config.h>
 
+#include "flint_table.h"
+
 #include <xapian/error.h>
 
 #include "safeerrno.h"
@@ -59,20 +63,19 @@ PREAD_PROTOTYPE
 PWRITE_PROTOTYPE
 #endif
 
-#include <stdio.h>
-#include <string.h>   /* for memmove */
-#include <limits.h>   /* for CHAR_BIT */
+#include <cstdio>    /* for rename */
+#include <cstring>   /* for memmove */
+#include <climits>   /* for CHAR_BIT */
 
-#include "flint_io.h"
-#include "flint_table.h"
-#include "flint_btreeutil.h"
 #include "flint_btreebase.h"
 #include "flint_cursor.h"
 #include "flint_utils.h"
 
+#include "debuglog.h"
+#include "io_utils.h"
 #include "omassert.h"
-#include "omdebug.h"
-#include <xapian/error.h>
+#include "str.h"
+#include "unaligned.h"
 #include "utils.h"
 
 #include <algorithm>  // for std::min()
@@ -221,7 +224,7 @@ void
 FlintTable::read_block(uint4 n, byte * p) const
 {
     // Log the value of p, not the contents of the block it points to...
-    DEBUGCALL(DB, void, "FlintTable::read_block", n << ", " << (void*)p);
+    LOGCALL_VOID(DB, "FlintTable::read_block", n | (void*)p);
     /* Use the base bit_map_size not the bitmap's size, because
      * the latter is uninitialised in readonly mode.
      */
@@ -237,11 +240,11 @@ FlintTable::read_block(uint4 n, byte * p) const
 	if (bytes_read == m) return;
 	if (bytes_read == -1) {
 	    if (errno == EINTR) continue;
-	    string message = "Error reading block " + om_tostring(n) + ": ";
+	    string message = "Error reading block " + str(n) + ": ";
 	    message += strerror(errno);
 	    throw Xapian::DatabaseError(message);
 	} else if (bytes_read == 0) {
-	    string message = "Error reading block " + om_tostring(n) + ": got end of file";
+	    string message = "Error reading block " + str(n) + ": got end of file";
 	    throw Xapian::DatabaseError(message);
 	} else if (bytes_read < m) {
 	    /* Read part of the block, which is not an error.  We should
@@ -259,7 +262,7 @@ FlintTable::read_block(uint4 n, byte * p) const
 	throw Xapian::DatabaseError(message);
     }
 
-    flint_io_read(handle, reinterpret_cast<char *>(p), block_size, block_size);
+    io_read(handle, reinterpret_cast<char *>(p), block_size, block_size);
 #endif
 }
 
@@ -272,7 +275,7 @@ FlintTable::read_block(uint4 n, byte * p) const
 void
 FlintTable::write_block(uint4 n, const byte * p) const
 {
-    DEBUGCALL(DB, void, "FlintTable::write_block", n << ", " << p);
+    LOGCALL_VOID(DB, "FlintTable::write_block", n | p);
     Assert(writable);
     /* Check that n is in range. */
     Assert(n / CHAR_BIT < base.get_bit_map_size());
@@ -322,7 +325,7 @@ FlintTable::write_block(uint4 n, const byte * p) const
 	throw Xapian::DatabaseError(message);
     }
 
-    flint_io_write(handle, reinterpret_cast<const char *>(p), block_size);
+    io_write(handle, reinterpret_cast<const char *>(p), block_size);
 #endif
 }
 
@@ -351,7 +354,7 @@ FlintTable::write_block(uint4 n, const byte * p) const
 void
 FlintTable::set_overwritten() const
 {
-    DEBUGCALL(DB, void, "FlintTable::set_overwritten", "");
+    LOGCALL_VOID(DB, "FlintTable::set_overwritten", NO_ARGS);
     // If we're writable, there shouldn't be another writer who could cause
     // overwritten to be flagged, so that's a DatabaseCorruptError.
     if (writable)
@@ -372,7 +375,7 @@ FlintTable::set_overwritten() const
 void
 FlintTable::block_to_cursor(Cursor_ * C_, int j, uint4 n) const
 {
-    DEBUGCALL(DB, void, "FlintTable::block_to_cursor", (void*)C_ << ", " << j << ", " << n);
+    LOGCALL_VOID(DB, "FlintTable::block_to_cursor", (void*)C_ | j | n);
     if (n == C_[j].n) return;
     byte * p = C_[j].p;
     Assert(p);
@@ -428,7 +431,7 @@ FlintTable::block_to_cursor(Cursor_ * C_, int j, uint4 n) const
 void
 FlintTable::alter()
 {
-    DEBUGCALL(DB, void, "FlintTable::alter", "");
+    LOGCALL_VOID(DB, "FlintTable::alter", NO_ARGS);
     Assert(writable);
 #ifdef DANGEROUS
     C[0].rewrite = true;
@@ -472,7 +475,7 @@ FlintTable::alter()
 
 int FlintTable::find_in_block(const byte * p, Key_ key, bool leaf, int c)
 {
-    DEBUGCALL_STATIC(DB, int, "FlintTable::find_in_block", reinterpret_cast<const void*>(p) << ", " << reinterpret_cast<const void*>(key.get_address()) << ", " << leaf << ", " << c);
+    LOGCALL_STATIC(DB, int, "FlintTable::find_in_block", reinterpret_cast<const void*>(p) | reinterpret_cast<const void*>(key.get_address()) | leaf | c);
     int i = DIR_START;
     if (leaf) i -= D2;
     int j = DIR_END(p);
@@ -502,7 +505,7 @@ int FlintTable::find_in_block(const byte * p, Key_ key, bool leaf, int c)
 bool
 FlintTable::find(Cursor_ * C_) const
 {
-    DEBUGCALL(DB, bool, "FlintTable::find", (void*)C_);
+    LOGCALL(DB, bool, "FlintTable::find", (void*)C_);
     // Note: the parameter is needed when we're called by FlintCursor
     const byte * p;
     int c;
@@ -536,7 +539,7 @@ FlintTable::find(Cursor_ * C_) const
 void
 FlintTable::compact(byte * p)
 {
-    DEBUGCALL(DB, void, "FlintTable::compact", (void*)p);
+    LOGCALL_VOID(DB, "FlintTable::compact", (void*)p);
     Assert(writable);
     int e = block_size;
     byte * b = buffer;
@@ -560,7 +563,7 @@ FlintTable::compact(byte * p)
 void
 FlintTable::split_root(uint4 split_n)
 {
-    DEBUGCALL(DB, void, "FlintTable::split_root", split_n);
+    LOGCALL_VOID(DB, "FlintTable::split_root", split_n);
     /* gain a level */
     ++level;
 
@@ -1016,10 +1019,10 @@ void FlintTable::form_key(const string & key) const
    deletions.
 */
 
-bool
+void
 FlintTable::add(const string &key, string tag, bool already_compressed)
 {
-    DEBUGCALL(DB, bool, "FlintTable::add", key << ", " << tag);
+    LOGCALL_VOID(DB, "FlintTable::add", key | tag);
     Assert(writable);
 
     if (handle < 0) create_and_open(block_size);
@@ -1097,7 +1100,8 @@ FlintTable::add(const string &key, string tag, bool already_compressed)
     /* FIXME: sort out this error higher up and turn this into
      * an assert.
      */
-    if (m >= BYTE_PAIR_RANGE) RETURN(false);
+    if (m >= BYTE_PAIR_RANGE)
+	throw Xapian::UnimplementedError("Can't handle insanely large tags");
 
     int n = 0; // initialise to shut off warning
 				      // - and there will be n to delete
@@ -1131,7 +1135,6 @@ FlintTable::add(const string &key, string tag, bool already_compressed)
 	cursor_created_since_last_modification = false;
 	++cursor_version;
     }
-    RETURN(true);
 }
 
 /* FlintTable::del(key) returns false if the key is not in the B-tree,
@@ -1143,10 +1146,15 @@ FlintTable::add(const string &key, string tag, bool already_compressed)
 bool
 FlintTable::del(const string &key)
 {
-    DEBUGCALL(DB, bool, "FlintTable::del", key);
+    LOGCALL(DB, bool, "FlintTable::del", key);
     Assert(writable);
 
-    if (handle < 0) RETURN(false);
+    if (handle < 0) {
+	if (handle == -2) {
+	    FlintTable::throw_database_closed();
+	}
+	RETURN(false);
+    }
 
     // We can't delete a key which we is too long for us to store.
     if (key.size() > FLINT_BTREE_MAX_KEY_LEN) RETURN(false);
@@ -1174,35 +1182,15 @@ FlintTable::del(const string &key)
 bool
 FlintTable::get_exact_entry(const string &key, string & tag) const
 {
-    DEBUGCALL(DB, bool, "FlintTable::get_exact_entry", key << ", " << tag);
+    LOGCALL(DB, bool, "FlintTable::get_exact_entry", key | tag);
     Assert(!key.empty());
 
-    if (handle < 0) RETURN(false);
-
-    // An oversized key can't exist, so attempting to search for it should fail.
-    if (key.size() > FLINT_BTREE_MAX_KEY_LEN) RETURN(false);
-
-    RETURN(find_tag(key, &tag));
-}
-
-bool
-FlintTable::key_exists(const string &key) const
-{
-    DEBUGCALL(DB, bool, "FlintTable::key_exists", key);
-    Assert(!key.empty());
-
-    // An oversized key can't exist, so attempting to search for it should fail.
-    if (key.size() > FLINT_BTREE_MAX_KEY_LEN) RETURN(false);
-
-    form_key(key);
-    RETURN(find(C));
-}
-
-bool
-FlintTable::find_tag(const string &key, string * tag) const
-{
-    DEBUGCALL(DB, bool, "FlintTable::find_tag", key << ", &tag");
-    if (handle == -1) RETURN(false);
+    if (handle < 0) {
+	if (handle == -2) {
+	    FlintTable::throw_database_closed();
+	}
+	RETURN(false);
+    }
 
     // An oversized key can't exist, so attempting to search for it should fail.
     if (key.size() > FLINT_BTREE_MAX_KEY_LEN) RETURN(false);
@@ -1210,10 +1198,23 @@ FlintTable::find_tag(const string &key, string * tag) const
     form_key(key);
     if (!find(C)) RETURN(false);
 
-    (void)read_tag(C, tag, false);
+    (void)read_tag(C, &tag, false);
     RETURN(true);
 }
 
+bool
+FlintTable::key_exists(const string &key) const
+{
+    LOGCALL(DB, bool, "FlintTable::key_exists", key);
+    Assert(!key.empty());
+
+    // An oversized key can't exist, so attempting to search for it should fail.
+    if (key.size() > FLINT_BTREE_MAX_KEY_LEN) RETURN(false);
+
+    form_key(key);
+    RETURN(find(C));
+}
+
 bool
 FlintTable::read_tag(Cursor_ * C_, string *tag, bool keep_compressed) const
 {
@@ -1263,7 +1264,7 @@ FlintTable::read_tag(Cursor_ * C_, string *tag, bool keep_compressed) const
 	inflate_zstream->avail_out = uInt(sizeof(buf));
 	err = inflate(inflate_zstream, Z_SYNC_FLUSH);
 	if (err == Z_BUF_ERROR && inflate_zstream->avail_in == 0) {
-	    DEBUGLINE(DB, "Z_BUF_ERROR - faking checksum of " << inflate_zstream->adler);
+	    LOGLINE(DB, "Z_BUF_ERROR - faking checksum of " << inflate_zstream->adler);
 	    Bytef header2[4];
 	    setint4(header2, 0, inflate_zstream->adler);
 	    inflate_zstream->next_in = header2;
@@ -1288,10 +1289,10 @@ FlintTable::read_tag(Cursor_ * C_, string *tag, bool keep_compressed) const
     }
     if (utag.size() != inflate_zstream->total_out) {
 	string msg = "compressed tag didn't expand to the expected size: ";
-	msg += om_tostring(utag.size());
+	msg += str(utag.size());
 	msg += " != ";
 	// OpenBSD's zlib.h uses off_t instead of uLong for total_out.
-	msg += om_tostring(size_t(inflate_zstream->total_out));
+	msg += str(size_t(inflate_zstream->total_out));
 	throw Xapian::DatabaseCorruptError(msg);
     }
 
@@ -1310,7 +1311,12 @@ FlintTable::set_full_compaction(bool parity)
 }
 
 FlintCursor * FlintTable::cursor_get() const {
-    if (handle < 0) return NULL;
+    if (handle < 0) {
+	if (handle == -2) {
+	    FlintTable::throw_database_closed();
+	}
+	return NULL;
+    }
     // FIXME Ick - casting away const is nasty
     return new FlintCursor(const_cast<FlintTable *>(this));
 }
@@ -1386,11 +1392,11 @@ FlintTable::basic_open(bool revision_supplied, flint_revision_number_t revision_
 	FlintTable_base *other_base = 0;
 
 	for (size_t i = 0; i < BTREE_BASES; ++i) {
-	    DEBUGLINE(UNKNOWN, "Checking (ch == " << ch << ") against "
-		      "basenames[" << i << "] == " << basenames[i]);
-	    DEBUGLINE(UNKNOWN, "bases[" << i << "].get_revision() == " <<
-		      bases[i].get_revision());
-	    DEBUGLINE(UNKNOWN, "base_ok[" << i << "] == " << base_ok[i]);
+	    LOGLINE(DB, "Checking (ch == " << ch << ") against "
+		    "basenames[" << i << "] == " << basenames[i]);
+	    LOGLINE(DB, "bases[" << i << "].get_revision() == " <<
+		    bases[i].get_revision());
+	    LOGLINE(DB, "base_ok[" << i << "] == " << base_ok[i]);
 	    if (ch == basenames[i]) {
 		basep = &bases[i];
 
@@ -1490,7 +1496,7 @@ FlintTable::do_open_to_write(bool revision_supplied,
 			     bool create_db)
 {
     if (handle == -2) {
-	throw Xapian::DatabaseError("Database has been closed");
+	FlintTable::throw_database_closed();
     }
     int flags = O_RDWR | O_BINARY;
     if (create_db) flags |= O_CREAT | O_TRUNC;
@@ -1545,9 +1551,10 @@ FlintTable::do_open_to_write(bool revision_supplied,
     return true;
 }
 
-FlintTable::FlintTable(const string & path_, bool readonly_,
-		       int compress_strategy_, bool lazy_)
-	: revision_number(0),
+FlintTable::FlintTable(const char * tablename_, const string & path_,
+		       bool readonly_, int compress_strategy_, bool lazy_)
+	: tablename(tablename_),
+	  revision_number(0),
 	  item_count(0),
 	  block_size(0),
 	  latest_revision_number(0),
@@ -1577,9 +1584,7 @@ FlintTable::FlintTable(const string & path_, bool readonly_,
 	  inflate_zstream(NULL),
 	  lazy(lazy_)
 {
-    DEBUGCALL(DB, void, "FlintTable::FlintTable",
-	      path_ << ", " << readonly_ << ", " <<
-	      compress_strategy_ << ", " << lazy_);
+    LOGCALL_VOID(DB, "FlintTable::FlintTable", tablename_ | path_ | readonly_ | compress_strategy_ | lazy_);
 }
 
 bool
@@ -1587,7 +1592,7 @@ FlintTable::really_empty() const
 {
     if (handle < 0) {
 	if (handle == -2) {
-	    throw Xapian::DatabaseError("Database has been closed");
+	    FlintTable::throw_database_closed();
 	}
 	return true;
     }
@@ -1625,7 +1630,7 @@ FlintTable::lazy_alloc_deflate_zstream() const {
 	if (deflate_zstream->msg) {
 	    msg += deflate_zstream->msg;
 	} else {
-	    msg += om_tostring(err);
+	    msg += str(err);
 	}
 	msg += ')';
 	delete deflate_zstream;
@@ -1662,7 +1667,7 @@ FlintTable::lazy_alloc_inflate_zstream() const {
 	if (inflate_zstream->msg) {
 	    msg += inflate_zstream->msg;
 	} else {
-	    msg += om_tostring(err);
+	    msg += str(err);
 	}
 	msg += ')';
 	delete inflate_zstream;
@@ -1673,7 +1678,7 @@ FlintTable::lazy_alloc_inflate_zstream() const {
 
 bool
 FlintTable::exists() const {
-    DEBUGCALL(DB, bool, "FlintTable::exists", "");
+    LOGCALL(DB, bool, "FlintTable::exists", NO_ARGS);
     return (file_exists(name + "DB") &&
 	    (file_exists(name + "baseA") || file_exists(name + "baseB")));
 }
@@ -1684,7 +1689,11 @@ FlintTable::exists() const {
 static void
 sys_unlink_if_exists(const string & filename)
 {
+#ifdef __WIN32__
+    if (msvc_posix_unlink(filename.c_str()) == -1) {
+#else
     if (unlink(filename) == -1) {
+#endif
 	if (errno == ENOENT) return;
 	throw Xapian::DatabaseError("Can't delete file: `" + filename +
 			      "': " + strerror(errno));
@@ -1694,7 +1703,7 @@ sys_unlink_if_exists(const string & filename)
 void
 FlintTable::erase()
 {
-    DEBUGCALL(DB, void, "FlintTable::erase", "");
+    LOGCALL_VOID(DB, "FlintTable::erase", NO_ARGS);
     close();
 
     sys_unlink_if_exists(name + "baseA");
@@ -1705,7 +1714,7 @@ FlintTable::erase()
 void
 FlintTable::set_block_size(unsigned int block_size_)
 {
-    DEBUGCALL(DB, void, "FlintTable::set_block_size", block_size_);
+    LOGCALL_VOID(DB, "FlintTable::set_block_size", block_size_);
     // Block size must in the range 2048..BYTE_PAIR_RANGE, and a power of two.
     if (block_size_ < 2048 || block_size_ > BYTE_PAIR_RANGE ||
 	(block_size_ & (block_size_ - 1)) != 0) {
@@ -1717,9 +1726,9 @@ FlintTable::set_block_size(unsigned int block_size_)
 void
 FlintTable::create_and_open(unsigned int block_size_)
 {
-    DEBUGCALL(DB, void, "FlintTable::create_and_open", block_size_);
+    LOGCALL_VOID(DB, "FlintTable::create_and_open", block_size_);
     if (handle == -2) {
-	throw Xapian::DatabaseError("Database has been closed");
+	FlintTable::throw_database_closed();
     }
     Assert(writable);
     close();
@@ -1739,7 +1748,7 @@ FlintTable::create_and_open(unsigned int block_size_)
     base_.set_block_size(block_size_);
     base_.set_have_fakeroot(true);
     base_.set_sequential(true);
-    base_.write_to_file(name + "baseA");
+    base_.write_to_file(name + "baseA", 'A', "", -1, NULL);
 
     /* remove the alternative base file, if any */
     sys_unlink_if_exists(name + "baseB");
@@ -1749,7 +1758,7 @@ FlintTable::create_and_open(unsigned int block_size_)
 }
 
 FlintTable::~FlintTable() {
-    DEBUGCALL(DB, void, "FlintTable::~FlintTable", "");
+    LOGCALL_VOID(DB, "FlintTable::~FlintTable", NO_ARGS);
     FlintTable::close();
 
     if (deflate_zstream) {
@@ -1768,7 +1777,7 @@ FlintTable::~FlintTable() {
 }
 
 void FlintTable::close(bool permanent) {
-    DEBUGCALL(DB, void, "FlintTable::close", "");
+    LOGCALL_VOID(DB, "FlintTable::close", NO_ARGS);
 
     if (handle >= 0) {
 	// If an error occurs here, we just ignore it, since we're just
@@ -1779,6 +1788,9 @@ void FlintTable::close(bool permanent) {
 
     if (permanent) {
 	handle = -2;
+	// Don't delete the resources in the table, since they may
+	// still be used to look up cached content.
+	return;
     }
 
     for (int j = level; j >= 0; j--) {
@@ -1795,26 +1807,17 @@ void FlintTable::close(bool permanent) {
 }
 
 void
-FlintTable::commit(flint_revision_number_t revision)
+FlintTable::flush_db()
 {
-    DEBUGCALL(DB, void, "FlintTable::commit", revision);
+    LOGCALL_VOID(DB, "FlintTable::flush_db", NO_ARGS);
     Assert(writable);
-
-    if (revision <= revision_number) {
-	throw Xapian::DatabaseError("New revision too low");
-    }
-
     if (handle < 0) {
-	latest_revision_number = revision_number = revision;
+	if (handle == -2) {
+	    FlintTable::throw_database_closed();
+	}
 	return;
     }
 
-    // FIXME: this doesn't work (probably because the table revisions get
-    // out of step) but it's wasteful to keep applying changes to value
-    // and position if they're never used...
-    //
-    // if (!Btree_modified) return;
-
     for (int j = level; j >= 0; j--) {
 	if (C[j].rewrite) {
 	    write_block(C[j].n, C[j].p);
@@ -1824,6 +1827,26 @@ FlintTable::commit(flint_revision_number_t revision)
     if (Btree_modified) {
 	faked_root_block = false;
     }
+}
+
+void
+FlintTable::commit(flint_revision_number_t revision, int changes_fd,
+		   const string * changes_tail)
+{
+    LOGCALL_VOID(DB, "FlintTable::commit", revision | changes_fd | changes_tail);
+    Assert(writable);
+
+    if (revision <= revision_number) {
+	throw Xapian::DatabaseError("New revision too low");
+    }
+
+    if (handle < 0) {
+	if (handle == -2) {
+	    FlintTable::throw_database_closed();
+	}
+	latest_revision_number = revision_number = revision;
+	return;
+    }
 
     try {
 	if (faked_root_block) {
@@ -1852,7 +1875,9 @@ FlintTable::commit(flint_revision_number_t revision)
 	    C[i].rewrite = false;
 	}
 
-	if (!flint_io_sync(handle)) {
+	// Do this as late as possible to allow maximum time for writes to be
+	// committed.
+	if (!io_sync(handle)) {
 	    (void)::close(handle);
 	    handle = -1;
 	    throw Xapian::DatabaseError("Can't commit new revision - failed to flush DB to disk");
@@ -1865,12 +1890,13 @@ FlintTable::commit(flint_revision_number_t revision)
 	string basefile = name;
 	basefile += "base";
 	basefile += char(base_letter);
-	base.write_to_file(tmp);
+	base.write_to_file(tmp, base_letter, tablename, changes_fd, changes_tail);
 #if defined __WIN32__
-	if (msvc_posix_rename(tmp.c_str(), basefile.c_str()) < 0) {
+	if (msvc_posix_rename(tmp.c_str(), basefile.c_str()) < 0)
 #else
-	if (rename(tmp.c_str(), basefile.c_str()) < 0) {
+	if (rename(tmp.c_str(), basefile.c_str()) < 0)
 #endif
+	{
 	    // With NFS, rename() failing may just mean that the server crashed
 	    // after successfully renaming, but before reporting this, and then
 	    // the retried operation fails.  So we need to check if the source
@@ -1898,13 +1924,57 @@ FlintTable::commit(flint_revision_number_t revision)
     }
 }
 
+void
+FlintTable::write_changed_blocks(int changes_fd)
+{
+    Assert(changes_fd >= 0);
+    if (handle < 0) return;
+    if (faked_root_block) return;
+
+    string buf;
+    buf += F_pack_uint(2u); // Indicate the item is a list of blocks
+    buf += F_pack_uint(strlen(tablename));
+    buf += tablename;
+    buf += F_pack_uint(block_size);
+    io_write(changes_fd, buf.data(), buf.size());
+
+    // Compare the old and new bitmaps to find blocks which have changed, and
+    // write them to the file descriptor.
+    uint4 n = 0;
+    byte * p = new byte[block_size];
+    try {
+	base.calculate_last_block();
+	while (base.find_changed_block(&n)) {
+	    buf = F_pack_uint(n + 1);
+	    io_write(changes_fd, buf.data(), buf.size());
+
+	    // Read block n.
+	    read_block(n, p);
+
+	    // Write block n to the file.
+	    io_write(changes_fd, reinterpret_cast<const char *>(p), block_size);
+	    ++n;
+	}
+	delete[] p;
+	p = 0;
+    } catch (...) {
+	delete[] p;
+	throw;
+    }
+    buf = F_pack_uint(0u);
+    io_write(changes_fd, buf.data(), buf.size());
+}
+
 void
 FlintTable::cancel()
 {
-    DEBUGCALL(DB, void, "FlintTable::cancel", "");
+    LOGCALL_VOID(DB, "FlintTable::cancel", NO_ARGS);
     Assert(writable);
 
     if (handle < 0) {
+	if (handle == -2) {
+	    FlintTable::throw_database_closed();
+	}
 	latest_revision_number = revision_number; // FIXME: we can end up reusing a revision if we opened a btree at an older revision, start to modify it, then cancel...
 	return;
     }
@@ -1927,6 +1997,8 @@ FlintTable::cancel()
 
     latest_revision_number = revision_number; // FIXME: we can end up reusing a revision if we opened a btree at an older revision, start to modify it, then cancel...
 
+    Btree_modified = false;
+
     for (int j = 0; j <= level; j++) {
 	C[j].n = BLK_UNUSED;
 	C[j].rewrite = false;
@@ -1944,7 +2016,7 @@ bool
 FlintTable::do_open_to_read(bool revision_supplied, flint_revision_number_t revision_)
 {
     if (handle == -2) {
-	throw Xapian::DatabaseError("Database has been closed");
+	FlintTable::throw_database_closed();
     }
     handle = ::open((name + "DB").c_str(), O_RDONLY | O_BINARY);
     if (handle < 0) {
@@ -1988,8 +2060,8 @@ FlintTable::do_open_to_read(bool revision_supplied, flint_revision_number_t revi
 void
 FlintTable::open()
 {
-    DEBUGCALL(DB, void, "FlintTable::open", "");
-    DEBUGLINE(DB, "opening at path " << name);
+    LOGCALL_VOID(DB, "FlintTable::open", NO_ARGS);
+    LOGLINE(DB, "opening at path " << name);
     close();
 
     if (!writable) {
@@ -2005,8 +2077,8 @@ FlintTable::open()
 bool
 FlintTable::open(flint_revision_number_t revision)
 {
-    DEBUGCALL(DB, bool, "FlintTable::open", revision);
-    DEBUGLINE(DB, "opening for particular revision at path " << name);
+    LOGCALL(DB, bool, "FlintTable::open", revision);
+    LOGLINE(DB, "opening for particular revision at path " << name);
     close();
 
     if (!writable) {
@@ -2065,6 +2137,7 @@ FlintTable::prev_for_sequential(Cursor_ * C_, int /*dummy*/) const
 	    } else {
 		read_block(n, p);
 	    }
+	    
 	    if (writable) AssertEq(revision_number, latest_revision_number);
 	    if (REVISION(p) > revision_number + writable) {
 		set_overwritten();
@@ -2118,7 +2191,6 @@ FlintTable::next_for_sequential(Cursor_ * C_, int /*dummy*/) const
 	    } else {
 		read_block(n, p);
 	    }
-
 	    if (writable) AssertEq(revision_number, latest_revision_number);
 	    if (REVISION(p) > revision_number + writable) {
 		set_overwritten();
@@ -2163,9 +2235,7 @@ FlintTable::next_default(Cursor_ * C_, int j) const
     c += D2;
     Assert((unsigned)c < block_size);
     // Sometimes c can be DIR_END(p) + 2 here it appears...
-    if (c > DIR_END(p)) c = DIR_END(p);
-    Assert(c <= DIR_END(p));
-    if (c == DIR_END(p)) {
+    if (c >= DIR_END(p)) {
 	if (j == level) return false;
 	if (!next_default(C_, j + 1)) return false;
 	c = DIR_START;
@@ -2181,6 +2251,12 @@ FlintTable::next_default(Cursor_ * C_, int j) const
     return true;
 }
 
+void
+FlintTable::throw_database_closed()
+{
+    throw Xapian::DatabaseError("Database has been closed");
+}
+
 /** Compares this key with key2.
 
    The result is true if this key precedes key2. The comparison is for byte
@@ -2198,7 +2274,7 @@ FlintTable::next_default(Cursor_ * C_, int j) const
 
 bool Key_::operator<(Key_ key2) const
 {
-    DEBUGCALL(DB, bool, "Key_::operator<", static_cast<const void*>(key2.p));
+    LOGCALL(DB, bool, "Key_::operator<", static_cast<const void*>(key2.p));
     int key1_len = length();
     int key2_len = key2.length();
     if (key1_len == key2_len) {
@@ -2221,7 +2297,7 @@ bool Key_::operator<(Key_ key2) const
 
 bool Key_::operator==(Key_ key2) const
 {
-    DEBUGCALL(DB, bool, "Key_::operator==", static_cast<const void*>(key2.p));
+    LOGCALL(DB, bool, "Key_::operator==", static_cast<const void*>(key2.p));
     int key1_len = length();
     if (key1_len != key2.length()) return false;
     // The keys are the same length, so we can compare the counts
