@@ -2,6 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010 Olly Betts
+ * Copyright 2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,18 +26,17 @@
 #include <xapian/error.h>
 #include <xapian/visibility.h>
 
-#include <algorithm>
-#include <string>
-using std::string;
-
 #include "flint_types.h"
 #include "flint_btreebase.h"
-#include "flint_btreeutil.h"
 #include "flint_cursor.h"
-#include "noreturn.h"
 
+#include "noreturn.h"
+#include "str.h"
 #include "stringutils.h"
-#include "utils.h"
+#include "unaligned.h"
+
+#include <algorithm>
+#include <string>
 
 #include <zlib.h>
 
@@ -97,7 +97,7 @@ class XAPIAN_VISIBILITY_DEFAULT Key_ {
 public:
     explicit Key_(const byte * p_) : p(p_) { }
     const byte * get_address() const { return p; }
-    void read(string * key) const {
+    void read(std::string * key) const {
 	key->assign(reinterpret_cast<const char *>(p + K1), length());
     }
     bool operator==(Key_ key2) const;
@@ -133,7 +133,7 @@ public:
 	return getint2(p, getK(p, I2) + I2);
     }
     Key_ key() const { return Key_(p + I2); }
-    void append_chunk(string * tag) const {
+    void append_chunk(std::string * tag) const {
 	/* number of bytes to extract from current component */
 	int cd = getK(p, I2) + I2 + C2;
 	int l = size() - cd;
@@ -178,9 +178,9 @@ public:
 	// Key size
 	setint1(p, I2, newsize - I2);
 	// Copy the main part of the key, possibly truncating.
-	memmove(p + I2 + K1, newkey.get_address() + K1, i);
+	std::memmove(p + I2 + K1, newkey.get_address() + K1, i);
 	// Copy the count part.
-	memmove(p + I2 + K1 + i, newkey.get_address() + K1 + newkey_len, C2);
+	std::memmove(p + I2 + K1 + i, newkey.get_address() + K1 + newkey_len, C2);
 	// Set tag contents to block number
 //	set_block_given_by(n);
 	setint4(p, newsize, n);
@@ -200,26 +200,26 @@ public:
 	set_key_len(K1);        /* null key */
 	set_size(I2 + K1 + 4);  /* total length */
     }
-    void form_key(const string & key_) {
-	string::size_type key_len = key_.length();
+    void form_key(const std::string & key_) {
+	std::string::size_type key_len = key_.length();
 	if (key_len > FLINT_BTREE_MAX_KEY_LEN) {
 	    // We check term length when a term is added to a document but
 	    // flint doubles zero bytes, so this can still happen for terms
 	    // which contain one or more zero bytes.
-	    string msg("Key too long: length was ");
-	    msg += om_tostring(key_len);
+	    std::string msg("Key too long: length was ");
+	    msg += str(key_len);
 	    msg += " bytes, maximum length of a key is "
 		   STRINGIZE(FLINT_BTREE_MAX_KEY_LEN) " bytes";
 	    throw Xapian::InvalidArgumentError(msg);
 	}
 
 	set_key_len(key_len + K1 + C2);
-	memmove(p + I2 + K1, key_.data(), key_len);
+	std::memmove(p + I2 + K1, key_.data(), key_len);
 	set_component_of(1);
     }
     // FIXME passing cd here is icky
     void set_tag(int cd, const char *start, int len, bool compressed) {
-	memmove(p + cd, start, len);
+	std::memmove(p + cd, start, len);
 	set_size(cd + len);
 	if (compressed) *p |= 0x80;
     }
@@ -277,6 +277,7 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable {
 	 *  This also does not open the table - either the create_and_open()
 	 *  or open() methods must be called before use is made of the table.
 	 *
+	 *  @param tablename_   The name of the table (used in changesets).
 	 *  @param path_	Path at which the table is stored.
 	 *  @param readonly_	whether to open the table for read only access.
 	 *  @param compress_strategy_	DONT_COMPRESS, Z_DEFAULT_STRATEGY,
@@ -284,8 +285,9 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable {
 	 *  @param lazy		If true, don't create the table until it's
 	 *			needed.
 	 */
-	FlintTable(const string & path_, bool readonly_,
-		   int compress_strategy_ = DONT_COMPRESS, bool lazy = false);
+	FlintTable(const char * tablename_, const std::string & path_,
+		   bool readonly_, int compress_strategy_ = DONT_COMPRESS,
+		   bool lazy = false);
 
 	/** Close the Btree.
 	 *
@@ -334,6 +336,13 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable {
 	 */
 	bool open(flint_revision_number_t revision_);
 
+	/** Flush any outstanding changes to the DB file of the table.
+	 *
+	 *  This must be called before commit, to ensure that the DB file is
+	 *  ready to be switched to a new version by the commit.
+	 */
+	void flush_db();
+
 	/** Commit any outstanding changes to the table.
 	 *
 	 *  Commit changes made by calling add() and del() to the Btree.
@@ -346,8 +355,18 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable {
 	 *          be greater than the latest revision number (see
 	 *          get_latest_revision_number()), or an exception will be
 	 *          thrown.
+	 *
+	 *  @param changes_fd  The file descriptor to write changes to.
+	 *	    Defaults to -1, meaning no changes will be written.
 	 */
-	void commit(flint_revision_number_t revision);
+	void commit(flint_revision_number_t revision, int changes_fd = -1,
+		    const std::string * changes_tail = NULL);
+
+	/** Append the list of blocks changed to a changeset file.
+	 *
+	 *  @param changes_fd  The file descriptor to write changes to.
+	 */
+	void write_changed_blocks(int changes_fd);
 
 	/** Cancel any outstanding changes.
 	 *
@@ -359,9 +378,10 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable {
 	/** Read an entry from the table, if and only if it is exactly that
 	 *  being asked for.
 	 *
-	 *  If the key is found in the table, the tag will be filled with
-	 *  the data associated with the key.  If the key is not found,
-	 *  the tag will be unmodified.
+	 *  If the key is found in the table, then the tag is copied to @a
+	 *  tag.  If the key is not found tag is left unchanged.
+	 * 
+	 *  The result is true iff the specified key is found in the Btree.
 	 *
 	 *  @param key  The key to look for in the table.
 	 *  @param tag  A tag object to fill with the value if found.
@@ -369,7 +389,7 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable {
 	 *  @return true if key is found in table,
 	 *          false if key is not found in table.
 	 */
-	bool get_exact_entry(const string & key, string & tag) const;
+	bool get_exact_entry(const std::string & key, std::string & tag) const;
 
 	/** Check if a key exists in the Btree.
 	 *
@@ -382,21 +402,7 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable {
 	 *  @return true if key is found in table,
 	 *          false if key is not found in table.
 	 */
-	bool key_exists(const string &key) const;
-
-	/** Find a key in the Btree and read its tag.
-	 *
-	 *  If the key is found the tag is copied to tag.  If the key is not
-	 *  found tag is left unchanged.
-	 * 
-	 *  The result is true iff the specified key is found in the Btree.
-	 *
-	 *  e.g.
-	 *
-	 *    string t;
-	 *    btree.find_tag("TODAY", &t); // get today's date
-	 */
-	bool find_tag(const string &key, string * tag) const;
+	bool key_exists(const std::string &key) const;
 
 	/** Read the tag value for the key pointed to by cursor C_.
 	 *
@@ -406,30 +412,25 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable {
 	 *  @return	true if current_tag holds compressed data (always
 	 *		false if keep_compressed was false).
 	 */
-	bool read_tag(Cursor_ * C_, string *tag, bool keep_compressed) const;
+	bool read_tag(Cursor_ * C_, std::string *tag, bool keep_compressed) const;
 
 	/** Add a key/tag pair to the table, replacing any existing pair with
 	 *  the same key.
 	 *
-	 *  If an error occurs during the operation, this will be signalled
-	 *  by a return value of false.  All modifications since the
-	 *  previous commit() will be lost.
+	 *  If an error occurs during the operation, an exception will be
+	 *  thrown.
 	 *
-	 *  If key is empty, then the null item is replaced.  If key.length()
-	 *  exceeds the limit on key size, false is returned.
+	 *  If key is empty, then the null item is replaced.
 	 *
-	 *  e.g.    ok = btree.add("TODAY", "Mon 9 Oct 2000");
+	 *  e.g.    btree.add("TODAY", "Mon 9 Oct 2000");
 	 *
 	 *  @param key   The key to store in the table.
 	 *  @param tag   The tag to store in the table.
 	 *  @param already_compressed	true if tag is already compressed,
 	 *		for example because it is being opaquely copied
 	 *		(default: false).
-	 *
-	 *  @return true if the operation completed successfully, false
-	 *          otherwise.
 	 */
-	bool add(const string &key, string tag, bool already_compressed = false);
+	void add(const std::string &key, std::string tag, bool already_compressed = false);
 
 	/** Delete an entry from the table.
 	 *
@@ -438,17 +439,15 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable {
 	 *  an empty key can't be removed, and false is returned.
 	 *
 	 *  If an error occurs during the operation, this will be signalled
-	 *  by a return value of false.  All modifications since the
-	 *  previous commit() will be lost.
+	 *  by an exception.
 	 *
-	 *  e.g.    ok = btree.del("TODAY")
+	 *  e.g.    bool deleted = btree.del("TODAY")
 	 *
 	 *  @param key   The key to remove from the table.
 	 *
-	 *  @return true if the operation completed successfully, false
-	 *          otherwise.
+	 *  @return true if an entry was removed; false if it did not exist.
 	 */
-	bool del(const string &key);
+	bool del(const std::string &key);
 
 	/// Erase this table from disk.
 	void erase();
@@ -601,7 +600,10 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable {
 	int add_kt(bool found);
 	void read_root();
 	void split_root(uint4 split_n);
-	void form_key(const string & key) const;
+	void form_key(const std::string & key) const;
+
+	/// The name of the table (used when writing changesets).
+	const char * tablename;
 
 	char other_base_letter() const {
 	   return (base_letter == 'A') ? 'B' : 'A';
@@ -666,7 +668,7 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable {
 	FlintTable_base base;
 
 	/// The path name of the B tree.
-	string name;
+	std::string name;
 
 	/** count of the number of successive instances of purely
 	 * sequential addition, starting at SEQ_START_POINT (neg) and
@@ -748,6 +750,9 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable {
 
 	/* Debugging methods */
 //	void report_block_full(int m, int n, const byte * p);
+
+        /// Throw an exception indicating that the database is closed.
+	XAPIAN_NORETURN(static void throw_database_closed());
 };
 
 #endif /* OM_HGUARD_FLINT_TABLE_H */
