@@ -4,7 +4,7 @@
 /*
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2007 Olly Betts
+ * Copyright 2002,2003,2004,2007,2009 Olly Betts
  * Copyright 2007 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -26,7 +26,7 @@
 #ifndef XAPIAN_INCLUDED_OUTPUT_H
 #define XAPIAN_INCLUDED_OUTPUT_H
 
-#include <fstream>
+#include <ostream>
 
 /// @Internal Helper macro for defining stream output of Xapian class.
 #define XAPIAN_OUTPUT_FUNCTION(CLASS) \
@@ -44,6 +44,7 @@ XAPIAN_OUTPUT_FUNCTION(Xapian::Document)
 
 #include <xapian/query.h>
 XAPIAN_OUTPUT_FUNCTION(Xapian::Query)
+XAPIAN_OUTPUT_FUNCTION(Xapian::Query::Internal)
 
 #include <xapian/enquire.h>
 XAPIAN_OUTPUT_FUNCTION(Xapian::RSet)
@@ -53,9 +54,6 @@ XAPIAN_OUTPUT_FUNCTION(Xapian::ESetIterator)
 XAPIAN_OUTPUT_FUNCTION(Xapian::ESet)
 XAPIAN_OUTPUT_FUNCTION(Xapian::Enquire)
 
-#include "stats.h"
-XAPIAN_OUTPUT_FUNCTION(Stats)
-
 #include <xapian/stem.h>
 XAPIAN_OUTPUT_FUNCTION(Xapian::Stem)
 
@@ -71,4 +69,12 @@ XAPIAN_OUTPUT_FUNCTION(Xapian::TermIterator)
 #include <xapian/valueiterator.h>
 XAPIAN_OUTPUT_FUNCTION(Xapian::ValueIterator)
 
+#include "replication.h"
+XAPIAN_OUTPUT_FUNCTION(Xapian::DatabaseMaster)
+XAPIAN_OUTPUT_FUNCTION(Xapian::DatabaseReplica)
+
+#include "weightinternal.h"
+XAPIAN_OUTPUT_FUNCTION(TermFreqs)
+XAPIAN_OUTPUT_FUNCTION(Xapian::Weight::Internal)
+
 #endif /* XAPIAN_INCLUDED_OUTPUT_H */
