@@ -294,6 +294,7 @@ MultiMatch::getorrecalc_maxweight(PostList *pl)
 	recalculate_w_max = false;
     } else {
 	wt = pl->get_maxweight();
+	LOGLINE(MATCH, "pl = (" << pl->get_description() << ")");
 	AssertEqDoubleParanoid(wt, pl->recalc_maxweight());
     }
     LOGLINE(MATCH, "max possible doc weight = " << wt);
@@ -313,7 +314,8 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
     AssertRel(check_at_least,>=,maxitems);
 
     if (!query) {
-	mset = Xapian::MSet(); // FIXME: mset.get_firstitem() will return 0 not first
+	mset = Xapian::MSet(new Xapian::MSet::Internal());
+	mset.internal->firstitem = first;
 	return;
     }
 
