@@ -2,7 +2,8 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2007 Olly Betts
+ * Copyright 2002,2003,2004,2007,2008,2009 Olly Betts
+ * Copyright 2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,12 +22,12 @@
  */
 
 #include <config.h>
-
 #include "phrasepostlist.h"
+
+#include "debuglog.h"
 #include "positionlist.h"
 #include "omassert.h"
-#include "omdebug.h"
-#include "utils.h"
+#include "str.h"
 
 #include <algorithm>
 
@@ -48,7 +49,7 @@ class PositionListCmpLt {
 bool
 NearPostList::test_doc()
 {
-    DEBUGCALL(MATCH, bool, "NearPostList::test_doc", "");
+    LOGCALL(MATCH, bool, "NearPostList::test_doc", NO_ARGS);
     std::vector<PositionList *> plists;
 
     std::vector<PostList *>::iterator i;
@@ -75,16 +76,16 @@ bool
 NearPostList::do_test(std::vector<PositionList *> &plists, Xapian::termcount i,
 		      Xapian::termcount min, Xapian::termcount max)
 {
-    DEBUGCALL(MATCH, bool, "NearPostList::do_test", "[plists], " << i << ", " << min << ", " << max);
-    DEBUGLINE(MATCH, "docid = " << get_docid() << ", window = " << window);
+    LOGCALL(MATCH, bool, "NearPostList::do_test", plists | i | min | max);
+    LOGLINE(MATCH, "docid = " << get_docid() << ", window = " << window);
     Xapian::termcount tmp = max + 1;
     // take care to avoid underflow
     if (window <= tmp) tmp -= window; else tmp = 0;
     plists[i]->skip_to(tmp);
     while (!plists[i]->at_end()) {
 	Xapian::termpos pos = plists[i]->get_position();
-	DEBUGLINE(MATCH, "[" << i << "]: " << max - window + 1 << " " << min
-		  << " " << pos << " " << max << " " << min + window - 1);
+	LOGLINE(MATCH, "[" << i << "]: " << max - window + 1 << " " << min <<
+		       " " << pos << " " << max << " " << min + window - 1);
 	if (pos > min + window - 1) RETURN(false);
 	if (i + 1 == plists.size()) RETURN(true);
 	if (pos < min) min = pos;
@@ -110,7 +111,7 @@ NearPostList::get_wdf() const
     // calculating the weight for synonym postlists, and it's not clear that
     // the above definition is exactly appropriate in this situation, anyway).
     //
-    // Instead, we could do an estimate of this value, based on the the lengths
+    // Instead, we could do an estimate of this value, based on the lengths
     // of the position lists.  Rather than having to open the position lists,
     // we could use the wdfs, which will be the same value unless the wdfs have
     // been artificially inflated - in which case we probably want to use the
@@ -149,10 +150,22 @@ NearPostList::get_wdf() const
     return std::max(wdf, Xapian::termcount(1));
 }
 
+TermFreqs
+NearPostList::get_termfreq_est_using_stats(
+	const Xapian::Weight::Internal & stats) const
+{
+    LOGCALL(MATCH, TermFreqs, "NearPostList::get_termfreq_est_using_stats", stats);
+    // No idea how to estimate this - FIXME
+    TermFreqs result(source->get_termfreq_est_using_stats(stats));
+    result.termfreq /= 2;
+    result.reltermfreq /= 2;
+    RETURN(result);
+}
+
 std::string
 NearPostList::get_description() const
 {
-    return "(Near " + om_tostring(window) + " " + source->get_description() + ")";
+    return "(Near " + str(window) + " " + source->get_description() + ")";
 }
 
 
@@ -162,7 +175,7 @@ NearPostList::get_description() const
 bool
 PhrasePostList::test_doc()
 {
-    DEBUGCALL(MATCH, bool, "PhrasePostList::test_doc", "");
+    LOGCALL(MATCH, bool, "PhrasePostList::test_doc", NO_ARGS);
     std::vector<PositionList *> plists;
 
     std::vector<PostList *>::iterator i;
@@ -181,7 +194,7 @@ PhrasePostList::test_doc()
     do {
 	plists[0]->next();
 	if (plists[0]->at_end()) {
-	    DEBUGLINE(MATCH, "--MISS--");
+	    LOGLINE(MATCH, "--MISS--");
 	    RETURN(false);
 	}
 	pos = plists[0]->get_position();
@@ -189,7 +202,7 @@ PhrasePostList::test_doc()
 	min = pos + plists.size() - idx;
 	if (min > window) min -= window; else min = 0;
     } while (!do_test(plists, 1, min, pos + window - idx));
-    DEBUGLINE(MATCH, "**HIT**");
+    LOGLINE(MATCH, "**HIT**");
     RETURN(true);
 }
 
@@ -197,35 +210,35 @@ bool
 PhrasePostList::do_test(std::vector<PositionList *> &plists, Xapian::termcount i,
 			Xapian::termcount min, Xapian::termcount max)
 {
-    DEBUGCALL(MATCH, bool, "PhrasePostList::do_test", "[plists],  " << i << ", " << min << ", " << max);
-    DEBUGLINE(MATCH, "docid = " << get_docid() << ", window = " << window);
+    LOGCALL(MATCH, bool, "PhrasePostList::do_test", plists | i | min | max);
+    LOGLINE(MATCH, "docid = " << get_docid() << ", window = " << window);
     Xapian::termpos idxi = plists[i]->index;
-    DEBUGLINE(MATCH, "my idx in phrase is " << idxi);
+    LOGLINE(MATCH, "my idx in phrase is " << idxi);
 
     Xapian::termpos mymin = min + idxi;
     Xapian::termpos mymax = max - plists.size() + idxi;
-    DEBUGLINE(MATCH, "MIN = " << mymin << " MAX = " << mymax);
+    LOGLINE(MATCH, "MIN = " << mymin << " MAX = " << mymax);
     // FIXME: this is worst case O(n^2) where n = length of phrase
     // Can we do better?
     for (Xapian::termcount j = 0; j < i; j++) {
 	Xapian::termpos idxj = plists[j]->index;
 	if (idxj > idxi) {
 	    Xapian::termpos tmp = plists[j]->get_position() + idxj - idxi;
-	    DEBUGLINE(MATCH, "ABOVE " << tmp);
+	    LOGLINE(MATCH, "ABOVE " << tmp);
 	    if (tmp < mymax) mymax = tmp;
 	} else {
 	    AssertRel(idxi, !=, idxj);
 	    Xapian::termpos tmp = plists[j]->get_position() + idxi - idxj;
-	    DEBUGLINE(MATCH, "BELOW " << tmp);
+	    LOGLINE(MATCH, "BELOW " << tmp);
 	    if (tmp > mymin) mymin = tmp;
 	}
-	DEBUGLINE(MATCH, "min = " << mymin << " max = " << mymax);
+	LOGLINE(MATCH, "min = " << mymin << " max = " << mymax);
     }
     plists[i]->skip_to(mymin);
 
     while (!plists[i]->at_end()) {
 	Xapian::termpos pos = plists[i]->get_position();
-	DEBUGLINE(MATCH, " " << mymin << " " << pos << " " << mymax);
+	LOGLINE(MATCH, " " << mymin << " " << pos << " " << mymax);
 	if (pos > mymax) RETURN(false);
 	if (i + 1 == plists.size()) RETURN(true);
 	Xapian::termpos tmp = pos + window - idxi;
@@ -244,7 +257,7 @@ PhrasePostList::do_test(std::vector<PositionList *> &plists, Xapian::termcount i
 Xapian::termcount
 PhrasePostList::get_wdf() const
 {
-    // Calculate the an estimate for the wdf of a phrase postlist.
+    // Calculate an estimate for the wdf of a phrase postlist.
     //
     // We use the minimum wdf of a sub-postlist as our estimate.  See the
     // comment in NearPostList::get_wdf for justification of this estimate.
@@ -264,9 +277,21 @@ PhrasePostList::get_wdf() const
     return std::max(wdf / 2, Xapian::termcount(1));
 }
 
+TermFreqs
+PhrasePostList::get_termfreq_est_using_stats(
+	const Xapian::Weight::Internal & stats) const
+{
+    LOGCALL(MATCH, TermFreqs, "PhrasePostList::get_termfreq_est_using_stats", stats);
+    // No idea how to estimate this - FIXME
+    TermFreqs result(source->get_termfreq_est_using_stats(stats));
+    result.termfreq /= 3;
+    result.reltermfreq /= 3;
+    RETURN(result);
+}
+
 std::string
 PhrasePostList::get_description() const
 {
-    return "(Phrase " + om_tostring(window) + " "
+    return "(Phrase " + str(window) + " "
 	   + source->get_description() + ")";
 }
