@@ -1,7 +1,7 @@
-/* @file serialise.cc
+/** @file serialise.cc
  * @brief functions to convert Xapian objects to strings and back
  */
-/* Copyright (C) 2006,2007,2010 Olly Betts
+/* Copyright (C) 2006,2007,2008,2009,2010 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,36 +30,14 @@
 #include "omenquireinternal.h"
 #include "serialise.h"
 #include "serialise-double.h"
-#include "stats.h"
 #include "utils.h"
+#include "weightinternal.h"
 
 #include <string>
-#include <string.h>
+#include <cstring>
 
 using namespace std;
 
-string
-encode_length(size_t len)
-{
-    string result;
-    if (len < 255) {
-	result += static_cast<unsigned char>(len);
-    } else {
-	result += '\xff';
-	len -= 255;
-	while (true) {
-	    unsigned char byte = static_cast<unsigned char>(len & 0x7f);
-	    len >>= 7;
-	    if (!len) {
-		result += (byte | static_cast<unsigned char>(0x80));
-		break;
-	    }
-	    result += byte;
-	}
-    }
-    return result;
-}
-
 size_t
 decode_length(const char ** p, const char *end, bool check_remaining)
 {
@@ -144,96 +122,58 @@ unserialise_error(const string &serialised_error, const string &prefix,
     throw Xapian::InternalError(newmsg, context);
 }
 
-string serialise_stats(const Stats &stats)
+string
+serialise_stats(const Xapian::Weight::Internal &stats)
 {
     string result;
 
+    result += encode_length(stats.total_length);
     result += encode_length(stats.collection_size);
     result += encode_length(stats.rset_size);
-    result += serialise_double(stats.average_length);
 
-    map<string, Xapian::doccount>::const_iterator i;
-
-    result += encode_length(stats.termfreq.size());
-    for (i = stats.termfreq.begin(); i != stats.termfreq.end(); ++i) {
+    result += encode_length(stats.termfreqs.size());
+    map<string, TermFreqs>::const_iterator i;
+    for (i = stats.termfreqs.begin(); i != stats.termfreqs.end(); ++i) {
 	result += encode_length(i->first.size());
 	result += i->first;
-	result += encode_length(i->second);
-    }
-
-    for (i = stats.reltermfreq.begin(); i != stats.reltermfreq.end(); ++i) {
-	result += encode_length(i->first.size());
-	result += i->first;
-	result += encode_length(i->second);
+	result += encode_length(i->second.termfreq);
+	if (stats.rset_size != 0)
+	    result += encode_length(i->second.reltermfreq);
     }
 
     return result;
 }
 
-Stats
+Xapian::Weight::Internal
 unserialise_stats(const string &s)
 {
     const char * p = s.data();
     const char * p_end = p + s.size();
 
-    Stats stat;
+    Xapian::Weight::Internal stat;
 
+    stat.total_length = decode_length(&p, p_end, false);
     stat.collection_size = decode_length(&p, p_end, false);
     stat.rset_size = decode_length(&p, p_end, false);
-    stat.average_length = unserialise_double(&p, p_end);
 
     size_t n = decode_length(&p, p_end, false);
     while (n--) {
 	size_t len = decode_length(&p, p_end, true);
 	string term(p, len);
 	p += len;
-	stat.termfreq.insert(make_pair(term, decode_length(&p, p_end, false)));
-    }
-
-    while (p != p_end) {
-	size_t len = decode_length(&p, p_end, true);
-	string term(p, len);
-	p += len;
-	stat.reltermfreq.insert(make_pair(term, decode_length(&p, p_end, false)));
+	Xapian::doccount termfreq(decode_length(&p, p_end, false));
+	if (stat.rset_size == 0) {
+	    stat.termfreqs.insert(make_pair(term, TermFreqs(termfreq, 0)));
+	} else {
+	    Xapian::doccount reltermfreq(decode_length(&p, p_end, false));
+	    stat.termfreqs.insert(make_pair(term,
+					    TermFreqs(termfreq, reltermfreq)));
+	}
     }
 
     return stat;
 }
 
-string
-serialise_mset_pre_30_5(const Xapian::MSet &mset)
-{
-    string result;
-
-    result += encode_length(mset.get_firstitem());
-    result += encode_length(mset.get_matches_lower_bound());
-    result += encode_length(mset.get_matches_estimated());
-    result += encode_length(mset.get_matches_upper_bound());
-    result += serialise_double(mset.get_max_possible());
-    result += serialise_double(mset.get_max_attained());
-    result += encode_length(mset.size());
-    for (Xapian::MSetIterator i = mset.begin(); i != mset.end(); ++i) {
-	result += serialise_double(i.get_weight());
-	result += encode_length(*i);
-	result += encode_length(i.get_collapse_key().size());
-	result += i.get_collapse_key();
-	result += encode_length(i.get_collapse_count());
-    }
-
-    const map<string, Xapian::MSet::Internal::TermFreqAndWeight> &termfreqandwts
-	= mset.internal->termfreqandwts;
-
-    map<string, Xapian::MSet::Internal::TermFreqAndWeight>::const_iterator j;
-    for (j = termfreqandwts.begin(); j != termfreqandwts.end(); ++j) {
-	result += encode_length(j->first.size());
-	result += j->first;
-	result += encode_length(j->second.termfreq);
-	result += serialise_double(j->second.termweight);
-    }
-
-    return result;
-}
-
 string
 serialise_mset(const Xapian::MSet &mset)
 {
@@ -243,6 +183,9 @@ serialise_mset(const Xapian::MSet &mset)
     result += encode_length(mset.get_matches_lower_bound());
     result += encode_length(mset.get_matches_estimated());
     result += encode_length(mset.get_matches_upper_bound());
+    result += encode_length(mset.get_uncollapsed_matches_lower_bound());
+    result += encode_length(mset.get_uncollapsed_matches_estimated());
+    result += encode_length(mset.get_uncollapsed_matches_upper_bound());
     result += serialise_double(mset.get_max_possible());
     result += serialise_double(mset.get_max_attained());
 
@@ -272,15 +215,15 @@ serialise_mset(const Xapian::MSet &mset)
 }
 
 Xapian::MSet
-unserialise_mset(const string &s)
+unserialise_mset(const char * p, const char * p_end)
 {
-    const char * p = s.data();
-    const char * p_end = p + s.size();
-
     Xapian::doccount firstitem = decode_length(&p, p_end, false);
     Xapian::doccount matches_lower_bound = decode_length(&p, p_end, false);
     Xapian::doccount matches_estimated = decode_length(&p, p_end, false);
     Xapian::doccount matches_upper_bound = decode_length(&p, p_end, false);
+    Xapian::doccount uncollapsed_lower_bound = decode_length(&p, p_end, false);
+    Xapian::doccount uncollapsed_estimated = decode_length(&p, p_end, false);
+    Xapian::doccount uncollapsed_upper_bound = decode_length(&p, p_end, false);
     Xapian::weight max_possible = unserialise_double(&p, p_end);
     Xapian::weight max_attained = unserialise_double(&p, p_end);
 
@@ -294,8 +237,8 @@ unserialise_mset(const string &s)
 	size_t len = decode_length(&p, p_end, true);
 	string key(p, len);
 	p += len;
-	items.push_back(Xapian::Internal::MSetItem(wt, did, key,
-						   decode_length(&p, p_end, false)));
+	Xapian::doccount collapse_cnt = decode_length(&p, p_end, false);
+	items.push_back(Xapian::Internal::MSetItem(wt, did, key, collapse_cnt));
     }
 
     map<string, Xapian::MSet::Internal::TermFreqAndWeight> terminfo;
@@ -314,6 +257,9 @@ unserialise_mset(const string &s)
 				       matches_upper_bound,
 				       matches_lower_bound,
 				       matches_estimated,
+				       uncollapsed_upper_bound,
+				       uncollapsed_lower_bound,
+				       uncollapsed_estimated,
 				       max_possible, max_attained,
 				       items, terminfo, percent_factor));
 }
