@@ -101,6 +101,12 @@ prefix_needs_colon(const string & prefix, unsigned ch)
 
 using Unicode::is_currency;
 
+inline bool
+is_positional(Xapian::Query::op op)
+{
+    return (op == Xapian::Query::OP_PHRASE || op == Xapian::Query::OP_NEAR);
+}
+
 /// A structure identifying a group of filter terms or a value range.
 struct filter_group_id {
     /** The prefix info for boolean filter terms.
@@ -382,8 +388,9 @@ Term::as_wildcarded_query(State * state_) const
 	    ++t;
 	}
     }
+    Query * q = new Query(Query::OP_SYNONYM, subqs.begin(), subqs.end());
     delete this;
-    return new Query(Query::OP_SYNONYM, subqs.begin(), subqs.end());
+    return q;
 }
 
 Query *
@@ -406,12 +413,13 @@ Term::as_partial_query(State * state_) const
 	// Add the term, as it would normally be handled, as an alternative.
 	subqs_full.push_back(Query(make_term(*piter), 1, pos));
     }
+    Query * q = new Query(Query::OP_OR,
+			  Query(Query::OP_SYNONYM,
+				subqs_partial.begin(), subqs_partial.end()),
+			  Query(Query::OP_SYNONYM,
+				subqs_full.begin(), subqs_full.end()));
     delete this;
-    return new Query(Query::OP_OR,
-		     Query(Query::OP_SYNONYM,
-			   subqs_partial.begin(), subqs_partial.end()),
-		     Query(Query::OP_SYNONYM,
-			   subqs_full.begin(), subqs_full.end()));
+    return q;
 }
 
 Query
@@ -935,10 +943,20 @@ just_had_operator_needing_term:
 			continue;
 		    }
 
+		    if (ch != ':') {
+			// Allow 'path:/usr/local' but not 'foo::bar::baz'.
+			while (is_phrase_generator(ch)) {
+			    if (++p == end)
+				goto not_prefix;
+			    ch = *p;
+			}
+		    }
+
 		    if (is_wordchar(ch)) {
 			// Prefixed term.
 			it = p;
 		    } else {
+not_prefix:
 			// It looks like a prefix but isn't, so parse it as
 			// text instead.
 			prefix_info = NULL;
@@ -1343,8 +1361,7 @@ reprocess:
 		}
 	    }
 	    Query q_original_terms;
-	    if (default_op == Query::OP_NEAR ||
-		default_op == Query::OP_PHRASE) {
+	    if (is_positional(default_op)) {
 		q_original_terms = Query(default_op,
 					 subqs2.begin(), subqs2.end(),
 					 subqs2.size() + 9);
@@ -1386,16 +1403,17 @@ reprocess:
 	goto reprocess;
     }
 
-    delete this;
-
-    if (subqs.empty()) return NULL;
-
-    if (default_op == Query::OP_NEAR ||
-	default_op == Query::OP_PHRASE) {
-	return new Query(default_op, subqs.begin(), subqs.end(),
-			 subqs.size() + 9);
+    Query * q = NULL;
+    if (!subqs.empty()) {
+	if (is_positional(default_op)) {
+	    q = new Query(default_op, subqs.begin(), subqs.end(),
+			     subqs.size() + 9);
+	} else {
+	    q = new Query(default_op, subqs.begin(), subqs.end());
+	}
     }
-    return new Query(default_op, subqs.begin(), subqs.end());
+    delete this;
+    return q;
 }
 
 /// Some terms which form a positional sub-query.
@@ -1530,7 +1548,7 @@ class Terms {
 	delete B;\
     } while (0)
 
-#line 1534 "queryparser/queryparser_internal.cc"
+#line 1552 "queryparser/queryparser_internal.cc"
 /* Next is all token values, in a form suitable for use by makeheaders.
 ** This section will be null unless lemon is run with the -m switch.
 */
@@ -1974,9 +1992,9 @@ static void yy_destructor(
     case 21: /* KET */
     case 22: /* EMPTY_GROUP_OK */
 {
-#line 1527 "queryparser/queryparser.lemony"
+#line 1545 "queryparser/queryparser.lemony"
 delete (yypminor->yy0);
-#line 1980 "queryparser/queryparser_internal.cc"
+#line 1998 "queryparser/queryparser_internal.cc"
 }
       break;
     case 25: /* expr */
@@ -1986,17 +2004,17 @@ delete (yypminor->yy0);
     case 31: /* stop_term */
     case 32: /* compound_term */
 {
-#line 1602 "queryparser/queryparser.lemony"
+#line 1620 "queryparser/queryparser.lemony"
 delete (yypminor->yy73);
-#line 1992 "queryparser/queryparser_internal.cc"
+#line 2010 "queryparser/queryparser_internal.cc"
 }
       break;
     case 28: /* prob */
     case 30: /* stop_prob */
 {
-#line 1697 "queryparser/queryparser.lemony"
+#line 1715 "queryparser/queryparser.lemony"
 delete (yypminor->yy12);
-#line 2000 "queryparser/queryparser_internal.cc"
+#line 2018 "queryparser/queryparser_internal.cc"
 }
       break;
     case 33: /* phrase */
@@ -2004,16 +2022,16 @@ delete (yypminor->yy12);
     case 36: /* near_expr */
     case 37: /* adj_expr */
 {
-#line 1899 "queryparser/queryparser.lemony"
+#line 1917 "queryparser/queryparser.lemony"
 (yypminor->yy46)->destroy();
-#line 2010 "queryparser/queryparser_internal.cc"
+#line 2028 "queryparser/queryparser_internal.cc"
 }
       break;
     case 35: /* group */
 {
-#line 1933 "queryparser/queryparser.lemony"
+#line 1951 "queryparser/queryparser.lemony"
 (yypminor->yy76)->destroy();
-#line 2017 "queryparser/queryparser_internal.cc"
+#line 2035 "queryparser/queryparser_internal.cc"
 }
       break;
     default:  break;   /* If no destructor action specified: do nothing */
@@ -2282,7 +2300,7 @@ static void yy_reduce(
   **     break;
   */
       case 0: /* query ::= expr */
-#line 1584 "queryparser/queryparser.lemony"
+#line 1602 "queryparser/queryparser.lemony"
 {
     // Save the parsed query in the State structure so we can return it.
     if (yymsp[0].minor.yy73) {
@@ -2292,30 +2310,30 @@ static void yy_reduce(
 	state->query = Query();
     }
 }
-#line 2296 "queryparser/queryparser_internal.cc"
+#line 2314 "queryparser/queryparser_internal.cc"
         break;
       case 1: /* query ::= */
-#line 1594 "queryparser/queryparser.lemony"
+#line 1612 "queryparser/queryparser.lemony"
 {
     // Handle a query string with no terms in.
     state->query = Query();
 }
-#line 2304 "queryparser/queryparser_internal.cc"
+#line 2322 "queryparser/queryparser_internal.cc"
         break;
       case 2: /* expr ::= prob_expr */
       case 9: /* bool_arg ::= expr */ yytestcase(yyruleno==9);
-#line 1605 "queryparser/queryparser.lemony"
+#line 1623 "queryparser/queryparser.lemony"
 { yygotominor.yy73 = yymsp[0].minor.yy73; }
-#line 2310 "queryparser/queryparser_internal.cc"
+#line 2328 "queryparser/queryparser_internal.cc"
         break;
       case 3: /* expr ::= bool_arg AND bool_arg */
-#line 1608 "queryparser/queryparser.lemony"
+#line 1626 "queryparser/queryparser.lemony"
 { BOOL_OP_TO_QUERY(yygotominor.yy73, yymsp[-2].minor.yy73, Query::OP_AND, yymsp[0].minor.yy73, "AND");   yy_destructor(yypParser,4,&yymsp[-1].minor);
 }
-#line 2316 "queryparser/queryparser_internal.cc"
+#line 2334 "queryparser/queryparser_internal.cc"
         break;
       case 4: /* expr ::= bool_arg NOT bool_arg */
-#line 1610 "queryparser/queryparser.lemony"
+#line 1628 "queryparser/queryparser.lemony"
 {
     // 'NOT foo' -> '<alldocuments> NOT foo'
     if (!yymsp[-2].minor.yy73 && (state->flags & QueryParser::FLAG_PURE_NOT)) {
@@ -2324,46 +2342,46 @@ static void yy_reduce(
     BOOL_OP_TO_QUERY(yygotominor.yy73, yymsp[-2].minor.yy73, Query::OP_AND_NOT, yymsp[0].minor.yy73, "NOT");
   yy_destructor(yypParser,5,&yymsp[-1].minor);
 }
-#line 2328 "queryparser/queryparser_internal.cc"
+#line 2346 "queryparser/queryparser_internal.cc"
         break;
       case 5: /* expr ::= bool_arg AND NOT bool_arg */
-#line 1619 "queryparser/queryparser.lemony"
+#line 1637 "queryparser/queryparser.lemony"
 { BOOL_OP_TO_QUERY(yygotominor.yy73, yymsp[-3].minor.yy73, Query::OP_AND_NOT, yymsp[0].minor.yy73, "AND NOT");   yy_destructor(yypParser,4,&yymsp[-2].minor);
   yy_destructor(yypParser,5,&yymsp[-1].minor);
 }
-#line 2335 "queryparser/queryparser_internal.cc"
+#line 2353 "queryparser/queryparser_internal.cc"
         break;
       case 6: /* expr ::= bool_arg AND HATE_AFTER_AND bool_arg */
-#line 1622 "queryparser/queryparser.lemony"
+#line 1640 "queryparser/queryparser.lemony"
 { BOOL_OP_TO_QUERY(yygotominor.yy73, yymsp[-3].minor.yy73, Query::OP_AND_NOT, yymsp[0].minor.yy73, "AND");   yy_destructor(yypParser,4,&yymsp[-2].minor);
   yy_destructor(yypParser,10,&yymsp[-1].minor);
 }
-#line 2342 "queryparser/queryparser_internal.cc"
+#line 2360 "queryparser/queryparser_internal.cc"
         break;
       case 7: /* expr ::= bool_arg OR bool_arg */
-#line 1625 "queryparser/queryparser.lemony"
+#line 1643 "queryparser/queryparser.lemony"
 { BOOL_OP_TO_QUERY(yygotominor.yy73, yymsp[-2].minor.yy73, Query::OP_OR, yymsp[0].minor.yy73, "OR");   yy_destructor(yypParser,2,&yymsp[-1].minor);
 }
-#line 2348 "queryparser/queryparser_internal.cc"
+#line 2366 "queryparser/queryparser_internal.cc"
         break;
       case 8: /* expr ::= bool_arg XOR bool_arg */
-#line 1628 "queryparser/queryparser.lemony"
+#line 1646 "queryparser/queryparser.lemony"
 { BOOL_OP_TO_QUERY(yygotominor.yy73, yymsp[-2].minor.yy73, Query::OP_XOR, yymsp[0].minor.yy73, "XOR");   yy_destructor(yypParser,3,&yymsp[-1].minor);
 }
-#line 2354 "queryparser/queryparser_internal.cc"
+#line 2372 "queryparser/queryparser_internal.cc"
         break;
       case 10: /* bool_arg ::= */
-#line 1637 "queryparser/queryparser.lemony"
+#line 1655 "queryparser/queryparser.lemony"
 {
     // Set the argument to NULL, which enables the bool_arg-using rules in
     // expr above to report uses of AND, OR, etc which don't have two
     // arguments.
     yygotominor.yy73 = NULL;
 }
-#line 2364 "queryparser/queryparser_internal.cc"
+#line 2382 "queryparser/queryparser_internal.cc"
         break;
       case 11: /* prob_expr ::= prob */
-#line 1649 "queryparser/queryparser.lemony"
+#line 1667 "queryparser/queryparser.lemony"
 {
     yygotominor.yy73 = yymsp[0].minor.yy12->query;
     yymsp[0].minor.yy12->query = NULL;
@@ -2401,45 +2419,45 @@ static void yy_reduce(
     }
     delete yymsp[0].minor.yy12;
 }
-#line 2405 "queryparser/queryparser_internal.cc"
+#line 2423 "queryparser/queryparser_internal.cc"
         break;
       case 12: /* prob_expr ::= term */
       case 30: /* stop_term ::= compound_term */ yytestcase(yyruleno==30);
       case 32: /* term ::= compound_term */ yytestcase(yyruleno==32);
-#line 1687 "queryparser/queryparser.lemony"
+#line 1705 "queryparser/queryparser.lemony"
 {
     yygotominor.yy73 = yymsp[0].minor.yy73;
 }
-#line 2414 "queryparser/queryparser_internal.cc"
+#line 2432 "queryparser/queryparser_internal.cc"
         break;
       case 13: /* prob ::= RANGE */
-#line 1699 "queryparser/queryparser.lemony"
+#line 1717 "queryparser/queryparser.lemony"
 {
     valueno slot = yymsp[0].minor.yy0->pos;
     const Query & range = yymsp[0].minor.yy0->as_value_range_query();
     yygotominor.yy12 = new ProbQuery;
     yygotominor.yy12->add_filter_range(slot, range);
 }
-#line 2424 "queryparser/queryparser_internal.cc"
+#line 2442 "queryparser/queryparser_internal.cc"
         break;
       case 14: /* prob ::= stop_prob RANGE */
-#line 1706 "queryparser/queryparser.lemony"
+#line 1724 "queryparser/queryparser.lemony"
 {
     valueno slot = yymsp[0].minor.yy0->pos;
     const Query & range = yymsp[0].minor.yy0->as_value_range_query();
     yygotominor.yy12 = yymsp[-1].minor.yy12;
     yygotominor.yy12->append_filter_range(slot, range);
 }
-#line 2434 "queryparser/queryparser_internal.cc"
+#line 2452 "queryparser/queryparser_internal.cc"
         break;
       case 15: /* prob ::= stop_term stop_term */
-#line 1713 "queryparser/queryparser.lemony"
+#line 1731 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = new ProbQuery;
     yygotominor.yy12->query = yymsp[-1].minor.yy73;
     if (yymsp[0].minor.yy73) {
 	Query::op op = state->default_op();
-	if (yygotominor.yy12->query && (op == Query::OP_NEAR || op == Query::OP_NEAR)) {
+	if (yygotominor.yy12->query && is_positional(op)) {
 	    // If default_op is OP_NEAR or OP_PHRASE, set the window size to
 	    // 11 for the first pair of terms and it will automatically grow
 	    // by one for each subsequent term.
@@ -2451,19 +2469,19 @@ static void yy_reduce(
 	}
     }
 }
-#line 2455 "queryparser/queryparser_internal.cc"
+#line 2473 "queryparser/queryparser_internal.cc"
         break;
       case 16: /* prob ::= prob stop_term */
-#line 1731 "queryparser/queryparser.lemony"
+#line 1749 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = yymsp[-1].minor.yy12;
     // If yymsp[0].minor.yy73 is a stopword, there's nothing to do here.
     if (yymsp[0].minor.yy73) add_to_query(yygotominor.yy12->query, state->default_op(), yymsp[0].minor.yy73);
 }
-#line 2464 "queryparser/queryparser_internal.cc"
+#line 2482 "queryparser/queryparser_internal.cc"
         break;
       case 17: /* prob ::= LOVE term */
-#line 1737 "queryparser/queryparser.lemony"
+#line 1755 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = new ProbQuery;
     if (state->default_op() == Query::OP_AND) {
@@ -2473,10 +2491,10 @@ static void yy_reduce(
     }
   yy_destructor(yypParser,8,&yymsp[-1].minor);
 }
-#line 2477 "queryparser/queryparser_internal.cc"
+#line 2495 "queryparser/queryparser_internal.cc"
         break;
       case 18: /* prob ::= stop_prob LOVE term */
-#line 1746 "queryparser/queryparser.lemony"
+#line 1764 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = yymsp[-2].minor.yy12;
     if (state->default_op() == Query::OP_AND) {
@@ -2489,66 +2507,66 @@ static void yy_reduce(
     }
   yy_destructor(yypParser,8,&yymsp[-1].minor);
 }
-#line 2493 "queryparser/queryparser_internal.cc"
+#line 2511 "queryparser/queryparser_internal.cc"
         break;
       case 19: /* prob ::= HATE term */
-#line 1758 "queryparser/queryparser.lemony"
+#line 1776 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = new ProbQuery;
     yygotominor.yy12->hate = yymsp[0].minor.yy73;
   yy_destructor(yypParser,9,&yymsp[-1].minor);
 }
-#line 2502 "queryparser/queryparser_internal.cc"
+#line 2520 "queryparser/queryparser_internal.cc"
         break;
       case 20: /* prob ::= stop_prob HATE term */
-#line 1763 "queryparser/queryparser.lemony"
+#line 1781 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = yymsp[-2].minor.yy12;
     add_to_query(yygotominor.yy12->hate, Query::OP_OR, yymsp[0].minor.yy73);
   yy_destructor(yypParser,9,&yymsp[-1].minor);
 }
-#line 2511 "queryparser/queryparser_internal.cc"
+#line 2529 "queryparser/queryparser_internal.cc"
         break;
       case 21: /* prob ::= HATE BOOLEAN_FILTER */
-#line 1768 "queryparser/queryparser.lemony"
+#line 1786 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = new ProbQuery;
     yygotominor.yy12->hate = new Query(yymsp[0].minor.yy0->get_query());
     delete yymsp[0].minor.yy0;
   yy_destructor(yypParser,9,&yymsp[-1].minor);
 }
-#line 2521 "queryparser/queryparser_internal.cc"
+#line 2539 "queryparser/queryparser_internal.cc"
         break;
       case 22: /* prob ::= stop_prob HATE BOOLEAN_FILTER */
-#line 1774 "queryparser/queryparser.lemony"
+#line 1792 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = yymsp[-2].minor.yy12;
     add_to_query(yygotominor.yy12->hate, Query::OP_OR, yymsp[0].minor.yy0->get_query());
     delete yymsp[0].minor.yy0;
   yy_destructor(yypParser,9,&yymsp[-1].minor);
 }
-#line 2531 "queryparser/queryparser_internal.cc"
+#line 2549 "queryparser/queryparser_internal.cc"
         break;
       case 23: /* prob ::= BOOLEAN_FILTER */
-#line 1780 "queryparser/queryparser.lemony"
+#line 1798 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = new ProbQuery;
     yygotominor.yy12->add_filter(yymsp[0].minor.yy0->get_filter_group_id(), yymsp[0].minor.yy0->get_query());
     delete yymsp[0].minor.yy0;
 }
-#line 2540 "queryparser/queryparser_internal.cc"
+#line 2558 "queryparser/queryparser_internal.cc"
         break;
       case 24: /* prob ::= stop_prob BOOLEAN_FILTER */
-#line 1786 "queryparser/queryparser.lemony"
+#line 1804 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = yymsp[-1].minor.yy12;
     yygotominor.yy12->append_filter(yymsp[0].minor.yy0->get_filter_group_id(), yymsp[0].minor.yy0->get_query());
     delete yymsp[0].minor.yy0;
 }
-#line 2549 "queryparser/queryparser_internal.cc"
+#line 2567 "queryparser/queryparser_internal.cc"
         break;
       case 25: /* prob ::= LOVE BOOLEAN_FILTER */
-#line 1792 "queryparser/queryparser.lemony"
+#line 1810 "queryparser/queryparser.lemony"
 {
     // LOVE BOOLEAN_FILTER(yymsp[0].minor.yy0) is just the same as BOOLEAN_FILTER
     yygotominor.yy12 = new ProbQuery;
@@ -2556,10 +2574,10 @@ static void yy_reduce(
     delete yymsp[0].minor.yy0;
   yy_destructor(yypParser,8,&yymsp[-1].minor);
 }
-#line 2560 "queryparser/queryparser_internal.cc"
+#line 2578 "queryparser/queryparser_internal.cc"
         break;
       case 26: /* prob ::= stop_prob LOVE BOOLEAN_FILTER */
-#line 1799 "queryparser/queryparser.lemony"
+#line 1817 "queryparser/queryparser.lemony"
 {
     // LOVE BOOLEAN_FILTER(yymsp[0].minor.yy0) is just the same as BOOLEAN_FILTER
     yygotominor.yy12 = yymsp[-2].minor.yy12;
@@ -2569,23 +2587,23 @@ static void yy_reduce(
     delete yymsp[0].minor.yy0;
   yy_destructor(yypParser,8,&yymsp[-1].minor);
 }
-#line 2573 "queryparser/queryparser_internal.cc"
+#line 2591 "queryparser/queryparser_internal.cc"
         break;
       case 27: /* stop_prob ::= prob */
-#line 1814 "queryparser/queryparser.lemony"
+#line 1832 "queryparser/queryparser.lemony"
 { yygotominor.yy12 = yymsp[0].minor.yy12; }
-#line 2578 "queryparser/queryparser_internal.cc"
+#line 2596 "queryparser/queryparser_internal.cc"
         break;
       case 28: /* stop_prob ::= stop_term */
-#line 1816 "queryparser/queryparser.lemony"
+#line 1834 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = new ProbQuery;
     yygotominor.yy12->query = yymsp[0].minor.yy73;
 }
-#line 2586 "queryparser/queryparser_internal.cc"
+#line 2604 "queryparser/queryparser_internal.cc"
         break;
       case 29: /* stop_term ::= TERM */
-#line 1830 "queryparser/queryparser.lemony"
+#line 1848 "queryparser/queryparser.lemony"
 {
     if (state->is_stopword(yymsp[0].minor.yy0)) {
 	yygotominor.yy73 = NULL;
@@ -2595,126 +2613,126 @@ static void yy_reduce(
     }
     delete yymsp[0].minor.yy0;
 }
-#line 2599 "queryparser/queryparser_internal.cc"
+#line 2617 "queryparser/queryparser_internal.cc"
         break;
       case 31: /* term ::= TERM */
-#line 1849 "queryparser/queryparser.lemony"
+#line 1867 "queryparser/queryparser.lemony"
 {
     yygotominor.yy73 = new Query(yymsp[0].minor.yy0->get_query_with_auto_synonyms());
     delete yymsp[0].minor.yy0;
 }
-#line 2607 "queryparser/queryparser_internal.cc"
+#line 2625 "queryparser/queryparser_internal.cc"
         break;
       case 33: /* compound_term ::= WILD_TERM */
-#line 1866 "queryparser/queryparser.lemony"
+#line 1884 "queryparser/queryparser.lemony"
 { yygotominor.yy73 = yymsp[0].minor.yy0->as_wildcarded_query(state); }
-#line 2612 "queryparser/queryparser_internal.cc"
+#line 2630 "queryparser/queryparser_internal.cc"
         break;
       case 34: /* compound_term ::= PARTIAL_TERM */
-#line 1869 "queryparser/queryparser.lemony"
+#line 1887 "queryparser/queryparser.lemony"
 { yygotominor.yy73 = yymsp[0].minor.yy0->as_partial_query(state); }
-#line 2617 "queryparser/queryparser_internal.cc"
+#line 2635 "queryparser/queryparser_internal.cc"
         break;
       case 35: /* compound_term ::= QUOTE phrase QUOTE */
-#line 1872 "queryparser/queryparser.lemony"
+#line 1890 "queryparser/queryparser.lemony"
 { yygotominor.yy73 = yymsp[-1].minor.yy46->as_phrase_query();   yy_destructor(yypParser,19,&yymsp[-2].minor);
   yy_destructor(yypParser,19,&yymsp[0].minor);
 }
-#line 2624 "queryparser/queryparser_internal.cc"
+#line 2642 "queryparser/queryparser_internal.cc"
         break;
       case 36: /* compound_term ::= phrased_term */
-#line 1875 "queryparser/queryparser.lemony"
+#line 1893 "queryparser/queryparser.lemony"
 { yygotominor.yy73 = yymsp[0].minor.yy46->as_phrase_query(); }
-#line 2629 "queryparser/queryparser_internal.cc"
+#line 2647 "queryparser/queryparser_internal.cc"
         break;
       case 37: /* compound_term ::= group */
-#line 1877 "queryparser/queryparser.lemony"
+#line 1895 "queryparser/queryparser.lemony"
 {
     yygotominor.yy73 = yymsp[0].minor.yy76->as_group(state);
 }
-#line 2636 "queryparser/queryparser_internal.cc"
+#line 2654 "queryparser/queryparser_internal.cc"
         break;
       case 38: /* compound_term ::= near_expr */
-#line 1882 "queryparser/queryparser.lemony"
+#line 1900 "queryparser/queryparser.lemony"
 { yygotominor.yy73 = yymsp[0].minor.yy46->as_near_query(); }
-#line 2641 "queryparser/queryparser_internal.cc"
+#line 2659 "queryparser/queryparser_internal.cc"
         break;
       case 39: /* compound_term ::= adj_expr */
-#line 1885 "queryparser/queryparser.lemony"
+#line 1903 "queryparser/queryparser.lemony"
 { yygotominor.yy73 = yymsp[0].minor.yy46->as_adj_query(); }
-#line 2646 "queryparser/queryparser_internal.cc"
+#line 2664 "queryparser/queryparser_internal.cc"
         break;
       case 40: /* compound_term ::= BRA expr KET */
-#line 1888 "queryparser/queryparser.lemony"
+#line 1906 "queryparser/queryparser.lemony"
 { yygotominor.yy73 = yymsp[-1].minor.yy73;   yy_destructor(yypParser,20,&yymsp[-2].minor);
   yy_destructor(yypParser,21,&yymsp[0].minor);
 }
-#line 2653 "queryparser/queryparser_internal.cc"
+#line 2671 "queryparser/queryparser_internal.cc"
         break;
       case 41: /* compound_term ::= SYNONYM TERM */
-#line 1890 "queryparser/queryparser.lemony"
+#line 1908 "queryparser/queryparser.lemony"
 {
     yygotominor.yy73 = new Query(yymsp[0].minor.yy0->get_query_with_synonyms());
     delete yymsp[0].minor.yy0;
   yy_destructor(yypParser,11,&yymsp[-1].minor);
 }
-#line 2662 "queryparser/queryparser_internal.cc"
+#line 2680 "queryparser/queryparser_internal.cc"
         break;
       case 42: /* phrase ::= TERM */
-#line 1901 "queryparser/queryparser.lemony"
+#line 1919 "queryparser/queryparser.lemony"
 {
     yygotominor.yy46 = new Terms;
     yygotominor.yy46->add_positional_term(yymsp[0].minor.yy0);
 }
-#line 2670 "queryparser/queryparser_internal.cc"
+#line 2688 "queryparser/queryparser_internal.cc"
         break;
       case 43: /* phrase ::= phrase TERM */
       case 45: /* phrased_term ::= phrased_term PHR_TERM */ yytestcase(yyruleno==45);
-#line 1906 "queryparser/queryparser.lemony"
+#line 1924 "queryparser/queryparser.lemony"
 {
     yygotominor.yy46 = yymsp[-1].minor.yy46;
     yygotominor.yy46->add_positional_term(yymsp[0].minor.yy0);
 }
-#line 2679 "queryparser/queryparser_internal.cc"
+#line 2697 "queryparser/queryparser_internal.cc"
         break;
       case 44: /* phrased_term ::= TERM PHR_TERM */
-#line 1918 "queryparser/queryparser.lemony"
+#line 1936 "queryparser/queryparser.lemony"
 {
     yygotominor.yy46 = new Terms;
     yygotominor.yy46->add_positional_term(yymsp[-1].minor.yy0);
     yygotominor.yy46->add_positional_term(yymsp[0].minor.yy0);
 }
-#line 2688 "queryparser/queryparser_internal.cc"
+#line 2706 "queryparser/queryparser_internal.cc"
         break;
       case 46: /* group ::= TERM GROUP_TERM */
-#line 1935 "queryparser/queryparser.lemony"
+#line 1953 "queryparser/queryparser.lemony"
 {
     yygotominor.yy76 = new TermGroup;
     yygotominor.yy76->add_term(yymsp[-1].minor.yy0);
     yygotominor.yy76->add_term(yymsp[0].minor.yy0);
 }
-#line 2697 "queryparser/queryparser_internal.cc"
+#line 2715 "queryparser/queryparser_internal.cc"
         break;
       case 47: /* group ::= group GROUP_TERM */
-#line 1941 "queryparser/queryparser.lemony"
+#line 1959 "queryparser/queryparser.lemony"
 {
     yygotominor.yy76 = yymsp[-1].minor.yy76;
     yygotominor.yy76->add_term(yymsp[0].minor.yy0);
 }
-#line 2705 "queryparser/queryparser_internal.cc"
+#line 2723 "queryparser/queryparser_internal.cc"
         break;
       case 48: /* group ::= group EMPTY_GROUP_OK */
-#line 1946 "queryparser/queryparser.lemony"
+#line 1964 "queryparser/queryparser.lemony"
 {
     yygotominor.yy76 = yymsp[-1].minor.yy76;
     yygotominor.yy76->set_empty_ok();
   yy_destructor(yypParser,22,&yymsp[0].minor);
 }
-#line 2714 "queryparser/queryparser_internal.cc"
+#line 2732 "queryparser/queryparser_internal.cc"
         break;
       case 49: /* near_expr ::= TERM NEAR TERM */
       case 51: /* adj_expr ::= TERM ADJ TERM */ yytestcase(yyruleno==51);
-#line 1957 "queryparser/queryparser.lemony"
+#line 1975 "queryparser/queryparser.lemony"
 {
     yygotominor.yy46 = new Terms;
     yygotominor.yy46->add_positional_term(yymsp[-2].minor.yy0);
@@ -2724,11 +2742,11 @@ static void yy_reduce(
 	delete yymsp[-1].minor.yy0;
     }
 }
-#line 2728 "queryparser/queryparser_internal.cc"
+#line 2746 "queryparser/queryparser_internal.cc"
         break;
       case 50: /* near_expr ::= near_expr NEAR TERM */
       case 52: /* adj_expr ::= adj_expr ADJ TERM */ yytestcase(yyruleno==52);
-#line 1967 "queryparser/queryparser.lemony"
+#line 1985 "queryparser/queryparser.lemony"
 {
     yygotominor.yy46 = yymsp[-2].minor.yy46;
     yygotominor.yy46->add_positional_term(yymsp[0].minor.yy0);
@@ -2737,7 +2755,7 @@ static void yy_reduce(
 	delete yymsp[-1].minor.yy0;
     }
 }
-#line 2741 "queryparser/queryparser_internal.cc"
+#line 2759 "queryparser/queryparser_internal.cc"
         break;
       default:
         break;
@@ -2766,11 +2784,11 @@ static void yy_parse_failed(
   while( !yypParser->yystack.empty() ) yy_pop_parser_stack(yypParser);
   /* Here code is inserted which will be executed whenever the
   ** parser fails */
-#line 1531 "queryparser/queryparser.lemony"
+#line 1549 "queryparser/queryparser.lemony"
 
     // If we've not already set an error message, set a default one.
     if (!state->error) state->error = "parse error";
-#line 2774 "queryparser/queryparser_internal.cc"
+#line 2792 "queryparser/queryparser_internal.cc"
   ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
 }
 #endif /* YYNOERRORRECOVERY */
@@ -2787,10 +2805,10 @@ static void yy_syntax_error(
   (void)yymajor;
   (void)yyminor;
 #define TOKEN (yyminor.yy0)
-#line 1536 "queryparser/queryparser.lemony"
+#line 1554 "queryparser/queryparser.lemony"
 
     yy_parse_failed(yypParser);
-#line 2794 "queryparser/queryparser_internal.cc"
+#line 2812 "queryparser/queryparser_internal.cc"
   ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
 }
 
