@@ -15,8 +15,8 @@
 #define WILD_TERM                      15
 #define PARTIAL_TERM                   16
 #define BOOLEAN_FILTER                 17
-#define RANGE_START                    18
-#define RANGE_END                      19
-#define QUOTE                          20
-#define BRA                            21
-#define KET                            22
+#define RANGE                          18
+#define QUOTE                          19
+#define BRA                            20
+#define KET                            21
+#define EMPTY_GROUP_OK                 22
