@@ -678,3 +678,67 @@ DEFINE_TESTCASE(failedreplace2, brass || chert || flint) {
     TEST_EQUAL(db.get_termfreq("foo"), 0);
     return true;
 }
+
+/// Coverage for SelectPostList::skip_to().
+DEFINE_TESTCASE(phrase3, positional) {
+    Xapian::Database db = get_database("apitest_phrase");
+
+    const char * phrase_words[] = { "phrase", "near" };
+    Xapian::Query q(Xapian::Query::OP_NEAR, phrase_words, phrase_words + 2, 12);
+    q = Xapian::Query(Xapian::Query::OP_AND_MAYBE, Xapian::Query("pad"), q);
+
+    Xapian::Enquire enquire(db);
+    enquire.set_query(q);
+    Xapian::MSet mset = enquire.get_mset(0, 5);
+
+    return true;
+}
+
+/// Check that get_mset(<large number>, 10) doesn't exhaust memory needlessly.
+// Regression test for fix in 1.2.4.
+DEFINE_TESTCASE(msetfirst2, backend) {
+    Xapian::Database db(get_database("apitest_simpledata"));
+    Xapian::Enquire enquire(db);
+    enquire.set_query(Xapian::Query("paragraph"));
+    Xapian::MSet mset;
+    // Before the fix, this tried to allocated too much memory.
+    mset = enquire.get_mset(0xfffffff0, 1);
+    TEST_EQUAL(mset.get_firstitem(), 0xfffffff0);
+    // Check that the number of documents gets clamped too.
+    mset = enquire.get_mset(1, 0xfffffff0);
+    TEST_EQUAL(mset.get_firstitem(), 1);
+    // Another regression test - MatchNothing used to give an MSet with
+    // get_firstitem() returning 0.
+    enquire.set_query(Xapian::Query::MatchNothing);
+    mset = enquire.get_mset(1, 1);
+    TEST_EQUAL(mset.get_firstitem(), 1);
+    return true;
+}
+
+DEFINE_TESTCASE(bm25weight2, backend) {
+    Xapian::Database db(get_database("etext"));
+    Xapian::Enquire enquire(db);
+    enquire.set_query(Xapian::Query("the"));
+    enquire.set_weighting_scheme(Xapian::BM25Weight(0, 0, 0, 0, 1));
+    Xapian::MSet mset = enquire.get_mset(0, 100);
+    TEST_REL(mset.size(),>=,2);
+    Xapian::weight weight0 = mset[0].get_weight();
+    for (size_t i = 1; i != mset.size(); ++i) {
+	TEST_EQUAL(weight0, mset[i].get_weight());
+    }
+    return true;
+}
+
+DEFINE_TESTCASE(tradweight2, backend) {
+    Xapian::Database db(get_database("etext"));
+    Xapian::Enquire enquire(db);
+    enquire.set_query(Xapian::Query("the"));
+    enquire.set_weighting_scheme(Xapian::TradWeight(0));
+    Xapian::MSet mset = enquire.get_mset(0, 100);
+    TEST_REL(mset.size(),>=,2);
+    Xapian::weight weight0 = mset[0].get_weight();
+    for (size_t i = 1; i != mset.size(); ++i) {
+	TEST_EQUAL(weight0, mset[i].get_weight());
+    }
+    return true;
+}
